package ardash.doodledudette;

import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.ui.ImageButton;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;

/**
 * A Button to show a part of a HSL adjustable color palette, which can alos
 * update the color of other Buttons.
 */
public class HSLButton extends ImageButton {
	private static TextureRegion whitePixelTextureRegion;
	private HSLColor hslColor;
	private Type type;

	public static enum Type {
		H, S, L
	}

	public HSLButton(Type t, HSLColor hlsColor) {
		super(createButtonStyle());
		type = t;
		setHslColor(hlsColor);
	}

	private static ImageButtonStyle createButtonStyle() {
		TextureRegionDrawable drawable = new TextureRegionDrawable(getWhitePixelTextureRegion());
		ImageButtonStyle style = new ImageButtonStyle(drawable, drawable, drawable, drawable, drawable, drawable);
		return style;
	}

	private static TextureRegion getWhitePixelTextureRegion() {
		if (whitePixelTextureRegion == null) {
			Pixmap whitePixmap = new Pixmap(1, 1, Pixmap.Format.RGBA8888);
			whitePixmap.setColor(1, 1, 1, 1);
			whitePixmap.fill();
			whitePixelTextureRegion = new TextureRegion(new Texture(whitePixmap));
			whitePixmap.dispose();
		}
		return whitePixelTextureRegion;
	}

	public static void disposeWhitePixelTextureRegion() {
		if (whitePixelTextureRegion != null) {
			whitePixelTextureRegion.getTexture().dispose();
			whitePixelTextureRegion = null;
		}
	}

	public Type getType() {
		return type;
	}

	public void setHslColor(HSLColor hslColor) {
		this.hslColor = new HSLColor(hslColor);
		setColor(hslColor.getRGBColor());
	}

	public void setH(float h) {
		hslColor.setH(h);
		setColor(hslColor.getRGBColor());
	}

	public void setS(float s) {
		hslColor.setS(s);
		setColor(hslColor.getRGBColor());
	}

	public void setL(float l) {
		hslColor.setL(l);
		setColor(hslColor.getRGBColor());
	}

	public float getH() {
		return hslColor.getH();
	}

	public float getS() {
		return hslColor.getS();
	}

	public float getL() {
		return hslColor.getL();
	}
}
