package ardash.doodledudette;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.zip.GZIPInputStream;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Pixmap.Format;
import com.badlogic.gdx.graphics.Texture;

public class SVGAssistant {

	private static final String TAG = "doodledudette.SVGAssistant";

	public static Texture loadGzippedSVGTexture(String svgFilePath, int canvasSize, boolean threshold) {
		try {
			FileHandle fh = Gdx.files.internal(svgFilePath);

			// Read gzipped SVG file content
			byte[] gzippedBytes = fh.readBytes();
			ByteArrayInputStream gzippedInputStream = new ByteArrayInputStream(gzippedBytes);
			GZIPInputStream gzipInputStream = new GZIPInputStream(gzippedInputStream);

			// Read uncompressed SVG content
			ByteArrayOutputStream svgOutputStream = new ByteArrayOutputStream();
			byte[] buffer = new byte[1024];
			int bytesRead;
			while ((bytesRead = gzipInputStream.read(buffer)) != -1) {
				svgOutputStream.write(buffer, 0, bytesRead);
			}
			gzipInputStream.close();

			final byte[] svgByteArray = svgOutputStream.toByteArray();
			// Convert SVG to PNG
			byte[] pngBytes = svgToPNG(canvasSize, svgByteArray);

			Pixmap pixmap = new Pixmap(pngBytes, 0, pngBytes.length);
			Texture texture = new Texture(pixmap);

			if (threshold) {
				applyThresholding(texture, 255);
			}

			return texture;
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	protected static byte[] svgToPNG(int canvasSize, final byte[] svgByteArray) {
		return GameManager.getInstance().getActionResolver().convertSvgBytesToPngBytes(svgByteArray, canvasSize,
				canvasSize);
	}

	public static void applyThresholding(Texture texture, int threshold) {
		Gdx.app.log(TAG, "start applyThresholding");
		// load texture into pixmap
		Pixmap pixmap = new Pixmap(texture.getWidth(), texture.getHeight(), Format.RGBA8888);
		if (!texture.getTextureData().isPrepared()) {
			texture.getTextureData().prepare();
		}
		pixmap.drawPixmap(texture.getTextureData().consumePixmap(), 0, 0);

		// apply thresholding
		for (int y = 0; y < pixmap.getHeight(); y++) {
			for (int x = 0; x < pixmap.getWidth(); x++) {
				int pixel = pixmap.getPixel(x, y);
				int r = (pixel & 0xff000000) >>> 24;
				int g = (pixel & 0x00ff0000) >>> 16;
				int b = (pixel & 0x0000ff00) >>> 8;

				// convert to grayscale
				int gray = (r + g + b) / 3;
				// apply thresholding
				if (gray >= threshold) {
					pixmap.setColor(1, 1, 1, 1); // white
				} else {
					pixmap.setColor(0, 0, 0, 1); // black
				}
				pixmap.drawPixel(x, y);
			}
		}

		// update texture with changed pixmap
		texture.draw(pixmap, 0, 0);

		// free pixmap memory
		pixmap.dispose();
		Gdx.app.log(TAG, "end applyThresholding");
	}
}
