package ardash.doodledudette.screens;

//import com.badlogic.gdx.Files;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.InputMultiplexer;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.Button;
import com.badlogic.gdx.scenes.scene2d.ui.ImageButton;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.scenes.scene2d.utils.SpriteDrawable;
import com.badlogic.gdx.utils.viewport.ScreenViewport;
import com.github.czyzby.kiwi.util.gdx.asset.Disposables;

import ardash.doodledudette.DDGame;
import ardash.doodledudette.HSLButton;
import ardash.doodledudette.HSLButton.Type;
import ardash.doodledudette.HSLColor;
import ardash.doodledudette.SVGAssistant;
import ardash.doodledudette.canvas.CanvasAggregate;

public class GameScreen implements Screen {
	private CanvasAggregate canvasAgg = new CanvasAggregate();

	// GUI
	private Stage stage;

	// Input multiplexer
	private InputMultiplexer multiplexer;

	HSLColor currentColor = new HSLColor(0f, 1f, 0.5f);
	private Button paintBtn1;
	private Button paintBtn2;
	private Button paintBtn3;

	// Arrays for buttons in each column
	private HSLButton[] hueButtons;
	private HSLButton[] saturationButtons;
	private HSLButton[] lightnessButtons;

	private static final float PADDING = 5f;
	private static final long DOUBLE_CLICK_THRESHOLD = 250; // Time in milliseconds

	public GameScreen(DDGame game, String svgzFile) {
		final int HEIGHT = Gdx.graphics.getHeight();
		final int WIDTH = Gdx.graphics.getWidth();
		final int CANVAS_SIZE = HEIGHT;
		float centimeterSizeInPixels = Gdx.graphics.getPpcX(); // Pixels per centimeter
		final float pencilBtnSize = centimeterSizeInPixels;

		// Create GUI stage
		stage = new Stage(new ScreenViewport()); // Same size as viewport for simplicity

		// Load button texture
		Texture btn1Texture = new Texture(Gdx.files.internal("btn1.png"));
		Texture btn2Texture = new Texture(Gdx.files.internal("btn2.png"));
		Texture btn3Texture = new Texture(Gdx.files.internal("btn3.png"));
		Texture btnShareTexture = SVGAssistant.loadGzippedSVGTexture("btnshare.svg.svgz", (int) pencilBtnSize, false);
		Texture btnMenuTexture = SVGAssistant.loadGzippedSVGTexture("btnback.svg.svgz", (int) pencilBtnSize, false);

		// Create button
		Button.ButtonStyle btn1Style = new Button.ButtonStyle();
		btn1Style.up = new SpriteDrawable(new Sprite(btn1Texture));
		Button.ButtonStyle btn2Style = new Button.ButtonStyle();
		btn2Style.up = new SpriteDrawable(new Sprite(btn2Texture));
		Button.ButtonStyle btn3Style = new Button.ButtonStyle();
		btn3Style.up = new SpriteDrawable(new Sprite(btn3Texture));

		ImageButton.ImageButtonStyle btnShareStyle = new ImageButton.ImageButtonStyle();
		btnShareStyle.imageUp = new SpriteDrawable(new Sprite(btnShareTexture));
		ImageButton.ImageButtonStyle btnMenuStyle = new ImageButton.ImageButtonStyle();
		btnMenuStyle.imageUp = new SpriteDrawable(new Sprite(btnMenuTexture));

		// Create GUI elements
		ImageButton shareBtn = new ImageButton(btnShareStyle);
		shareBtn.setPosition(WIDTH - shareBtn.getWidth() - PADDING, shareBtn.getHeight());
		stage.addActor(shareBtn);
		shareBtn.addListener(new ClickListener() {
			private long lastClickTime;
			private int clickCount;

			@Override
			public void clicked(InputEvent event, float x, float y) {
				clickCount++;
				long currentTime = System.currentTimeMillis();
				if (currentTime - lastClickTime < DOUBLE_CLICK_THRESHOLD) {
					// Double-click detected
					game.getActionResolver().shareImage(canvasAgg.getCanvasTexture());
					clickCount = 1;
				}
				lastClickTime = currentTime;
				if (clickCount%4==0) {
					game.getActionResolver().showToast("double-click to share");
				}
			}
		});

		ImageButton menuBtn = new ImageButton(btnMenuStyle);
		menuBtn.setPosition(WIDTH - menuBtn.getWidth() - PADDING, 0);
		stage.addActor(menuBtn);
		menuBtn.addListener(new ClickListener() {
			private long lastClickTime;
			private int clickCount;

			@Override
			public void clicked(InputEvent event, float x, float y) {
				clickCount++;
				long currentTime = System.currentTimeMillis();
				if (currentTime - lastClickTime < DOUBLE_CLICK_THRESHOLD) {
					// Double-click detected
					game.setScreen(new LoadingScreen() {
						@Override
						public void doAfterShow() {
							game.setScreen(new PictureSelectionScreen(game));
							clickCount = 1;
						}
					});
				}
				lastClickTime = currentTime;
				if (clickCount%4==0) {
					game.getActionResolver().showToast("double-click to go back");
				}
			}
		});

		canvasAgg.setRadius(pencilBtnSize / 2);
		paintBtn1 = new Button(btn1Style);
		paintBtn1.setColor(Color.RED);
		paintBtn1.setSize(pencilBtnSize, pencilBtnSize);
		paintBtn1.setPosition(WIDTH - paintBtn1.getWidth() - PADDING, HEIGHT - paintBtn1.getHeight() - PADDING);
		paintBtn1.addListener(new ClickListener() {
			@Override
			public void enter(InputEvent event, float x, float y, int pointer, Actor fromActor) {
				super.enter(event, x, y, pointer, fromActor);
				canvasAgg.setRadius(pencilBtnSize / 2);
				System.out.println("Button1 " + " enter!");
			}
		});
		stage.addActor(paintBtn1);
		paintBtn1.setColor(Color.RED);

		paintBtn2 = new Button(btn2Style);
		paintBtn2.setColor(Color.RED);
		paintBtn2.setSize(pencilBtnSize, pencilBtnSize);
		paintBtn2.setPosition(WIDTH - paintBtn1.getWidth() - PADDING, HEIGHT - (paintBtn1.getHeight() + PADDING) * 2);
		paintBtn2.addListener(new ClickListener() {
			@Override
			public void enter(InputEvent event, float x, float y, int pointer, Actor fromActor) {
				super.enter(event, x, y, pointer, fromActor);
				canvasAgg.setRadius(pencilBtnSize / 4);
				System.out.println("Button2 " + " enter!");
			}
		});
		stage.addActor(paintBtn2);

		paintBtn3 = new Button(btn3Style);
		paintBtn3.setColor(Color.RED);
		paintBtn3.setSize(pencilBtnSize, pencilBtnSize);
		paintBtn3.setPosition(WIDTH - paintBtn1.getWidth() - PADDING, HEIGHT - (paintBtn1.getHeight() + PADDING) * 3);
		paintBtn3.addListener(new ClickListener() {
			@Override
			public void enter(InputEvent event, float x, float y, int pointer, Actor fromActor) {
				super.enter(event, x, y, pointer, fromActor);
				canvasAgg.setRadius(pencilBtnSize / 8);
				System.out.println("Button3 " + " enter!");
			}
		});
		stage.addActor(paintBtn3);

		// Initialize arrays
		hueButtons = new HSLButton[20];
		saturationButtons = new HSLButton[20];
		lightnessButtons = new HSLButton[20];

		float startHue = 0f;
		float endHue = 360f - 18f; // 360/20 = stepsize 18 . So remove 18.
		float hueStep = (endHue - startHue) / 19f; // 20 buttons in total, including start and end colors

		float buttonHeight = HEIGHT / 20f; // Calculate the height of each button dynamically

		// calculate width for color buttons
		final float availableWidthFoColorPalette = WIDTH - CANVAS_SIZE - pencilBtnSize - PADDING * 3;
		final float minColBtnWidth = buttonHeight;
//        final float maxColBtnWidth = availableWidthFoColorPalette/3f;
		final float maxColBtnWidth = minColBtnWidth * 3f;
		float btnWidth = availableWidthFoColorPalette / 3f;
		btnWidth = MathUtils.clamp(btnWidth, minColBtnWidth, maxColBtnWidth);
		// Create buttons for each column
		for (int i = 0; i < 20; i++) {
			// Interpolate color for hue column
			float hue = startHue + hueStep * i;
			HSLColor hueColor = new HSLColor(hue, 1f, 0.5f);
			HSLColor satColor = new HSLColor(0f, MathUtils.lerp(0.05f, 1f, i / 20f), 0.5f);
			HSLColor ligColor = new HSLColor(0f, 1f, MathUtils.lerp(0.05f, 1f, i / 20f));

			// Create hue button
			hueButtons[i] = createColoredButton(Type.H, btnWidth, buttonHeight, CANVAS_SIZE + PADDING,
					HEIGHT - buttonHeight * (i + 1), "HueBtn" + i, hueColor);

			// Create saturation button
			saturationButtons[i] = createColoredButton(Type.S, btnWidth, buttonHeight, CANVAS_SIZE + PADDING + btnWidth,
					HEIGHT - buttonHeight * (i + 1), "SatBtn" + i, satColor);

			// Create lightness button
			lightnessButtons[i] = createColoredButton(Type.L, btnWidth, buttonHeight,
					CANVAS_SIZE + PADDING + btnWidth * 2, HEIGHT - buttonHeight * (i + 1), "LigBtn" + i, ligColor);
		}

		// Setup input multiplexer
		canvasAgg.createInputProcessor();

		multiplexer = new InputMultiplexer();
		multiplexer.addProcessor(stage);
		multiplexer.addProcessor(canvasAgg.getInputProcessor());
		Gdx.input.setInputProcessor(multiplexer);

		// long process
		canvasAgg.buildCanvas(svgzFile);
	}

	private HSLButton createColoredButton(Type type, float width, float height, float x, float y, String name,
			HSLColor color) {
		// Create button
		HSLButton button = new HSLButton(type, color);
		button.setSize(width, height);
		button.setPosition(x, y);
		button.addListener(new ClickListener() {
			@Override
			public void enter(InputEvent event, float x, float y, int pointer, Actor fromActor) {

				// if the drag came from the canvas, do nothing here. (Bugfix for #13)
				if (canvasAgg.isDragging()) {
					return;
				}

				Gdx.app.log(name, "Clicked!a");
				currentColor.setH(button.getH());
				currentColor.setS(button.getS());
				currentColor.setL(button.getL());
				paintBtn1.setColor(currentColor.getRGBColor());
				paintBtn2.setColor(currentColor.getRGBColor());
				paintBtn3.setColor(currentColor.getRGBColor());
				canvasAgg.setCurrentColor(currentColor);

				// Update saturation buttons
				if (button.getType().equals(Type.H) || button.getType().equals(Type.L)) {
					for (int i = 0; i < 20; i++) {
						saturationButtons[i].setH(currentColor.getH());
					}
				}

				// Update lightness buttons
				if (button.getType().equals(Type.H) || button.getType().equals(Type.S)) {
					for (int i = 0; i < 20; i++) {
						lightnessButtons[i].setH(currentColor.getH());
						lightnessButtons[i].setS(currentColor.getS());
					}
				}
			}
		});
		stage.addActor(button);
		return button;
	}

	@Override
	public void show() {
		// this method must be fast. all the heavy work must be done in the constructor,
		// because as long as the constructor runs, the loading screen will be shown
	}

	@Override
	public void render(float delta) {
		// Clear the screen
		Gdx.gl.glClearColor(0, 0, 0, 1); // Set background color to black
		Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);

		// Render the canvas texture
		canvasAgg.renderCanvasTexture();

		stage.act(Math.min(Gdx.graphics.getDeltaTime(), 1 / 30f));
		stage.draw();

	}

	@Override
	public void dispose() {
		// TODO dispose button textures
		Disposables.gracefullyDisposeOf(canvasAgg, stage);
	}

	@Override
	public void resize(int width, int height) {
		stage.getViewport().update(width, height, true);
	}

	@Override
	public void pause() {
		// TODO Auto-generated method stub

	}

	@Override
	public void resume() {
		// TODO Auto-generated method stub

	}

	@Override
	public void hide() {
		// we show this screen only once, when we hide it, we also dispose it
		dispose();
	}
}
