package ardash.doodledudette.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;

public abstract class LoadingScreen implements Screen {
	private SpriteBatch batch;
	private BitmapFont font;
	private float age = 0f;

	public LoadingScreen() {
	}

	/**
	 * This will be done, after the loading screen has been shown for a little
	 * while. If the content in here takes a longer time, it will block the render
	 * thread, but the loading screen will still be shown (still, not animated).
	 */
	public abstract void doAfterShow();

	@Override
	public void show() {
		batch = new SpriteBatch();
		font = new BitmapFont();
		font.getData().setScale(4); // adjust scale as needed
	}

	@Override
	public void render(float delta) {
		age += delta;
		Gdx.gl.glClearColor(0, 0, 0, 1);
		Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);

		batch.begin();
		font.draw(batch, "Loading...", Gdx.graphics.getWidth() / 2f - 50f, Gdx.graphics.getHeight() / 2f);
		batch.end();

		if (age <= 0.2f) {
			return;
		}

		doAfterShow();
	}

	@Override
	public void resize(int width, int height) {
	}

	@Override
	public void pause() {
	}

	@Override
	public void resume() {
	}

	@Override
	public void hide() {
		// we show this screen only once, when we hide it, we also dispose it
		dispose();
	}

	@Override
	public void dispose() {
		batch.dispose();
		font.dispose();
	}
}
