package ardash.doodledudette.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.ScreenAdapter;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.actions.Actions;
import com.badlogic.gdx.scenes.scene2d.actions.DelayAction;
import com.badlogic.gdx.scenes.scene2d.actions.RepeatAction;
import com.badlogic.gdx.scenes.scene2d.actions.RunnableAction;
import com.badlogic.gdx.scenes.scene2d.actions.SequenceAction;
import com.badlogic.gdx.scenes.scene2d.ui.HorizontalGroup;
import com.badlogic.gdx.scenes.scene2d.ui.ImageButton;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.utils.ClickListener;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;
import com.badlogic.gdx.utils.viewport.FitViewport;
import com.badlogic.gdx.utils.viewport.Viewport;

import ardash.doodledudette.DDGame;
import ardash.doodledudette.SVGAssistant;

public class PictureSelectionScreen extends ScreenAdapter {

    private static final int NUMBER_OF_PICTURE_PAGES = 5;
	private Stage stage;
    private OrthographicCamera camera;
    private Viewport viewport;
    private HorizontalGroup horizontalGroup;
    private ScrollPane scrollPane;
//    private int currentGroupIndex = 0;
//    private int totalGroups = 3; // Number of groups
    private float lifetime = 0f;

    public PictureSelectionScreen(final DDGame game) {
        camera = new OrthographicCamera();
        viewport = new FitViewport(800, 480, camera); // adjust the width and height as per your needs
        stage = new Stage(viewport);
//        stage.setDebugAll(true);
        Gdx.input.setInputProcessor(stage);

        horizontalGroup = new HorizontalGroup();
        horizontalGroup.space(200); // Set padding between items

	     // Add padding or empty elements on the left and right sides of the HorizontalGroup
	     float padding = 200; // Adjust padding as needed
	     Actor leftPadding = new Actor();
	     leftPadding.setWidth(padding);
	     horizontalGroup.addActor(leftPadding);
	
	     // Add tables with buttons
	     for (int i = 0; i < NUMBER_OF_PICTURE_PAGES; i++) {
	         Table table = createButtonTable(i, game);
	         horizontalGroup.addActor(table);
	     }
	
	     Actor rightPadding = new Actor();
	     rightPadding.setWidth(padding);
	     horizontalGroup.addActor(rightPadding);
	
	     scrollPane = new ScrollPane(horizontalGroup);
	     scrollPane.setScrollingDisabled(false, true); // Allow scrolling only in horizontal direction
	     scrollPane.setFillParent(true);
	
	     stage.addActor(scrollPane);

    }

    private Table createButtonTable(int series, final DDGame game) {
        Table table = new Table();

        // Add buttons with SVG textures
        for (int i = 0; i < 10; i++) {
            final String svgFilePath = series + "_" + i + ".svg.svgz";
            Texture texture = SVGAssistant.loadGzippedSVGTexture(svgFilePath, 100, false);
            ImageButton.ImageButtonStyle style = new ImageButton.ImageButtonStyle();
            style.imageUp = new TextureRegionDrawable(new TextureRegion(texture));

            ImageButton button = new ImageButton(style);
            table.add(button).pad(10);

            button.addListener(new ClickListener() {
                @Override
                public void clicked(InputEvent event, float x, float y) {
                    super.clicked(event, x, y);
                    game.setScreen(new LoadingScreen() {
                        @Override
                        public void doAfterShow() {
                            game.setScreen(new GameScreen(game, svgFilePath));
                        }
                    });
                }
            });

            if ((i + 1) % 5 == 0) {
                table.row();
            }
        }

        return table;
    }
    
    @Override
    public void show() {
        super.show();
        addScrollAffordance();
    }

    private void addScrollAffordance() {
        RunnableAction scrollABitRight = Actions.run(() -> scrollPane.setScrollX(scrollPane.getScrollX() + 50f));
        RunnableAction scrollABitRight2 = Actions.run(() -> scrollPane.setScrollX(scrollPane.getScrollX() + 50f));
        RunnableAction scrollABitLeft = Actions.run(() -> scrollPane.setScrollX(scrollPane.getScrollX() - 100f));
        DelayAction d1 = Actions.delay(3f);
        DelayAction d2 = Actions.delay(0.2f);
        DelayAction d3 = Actions.delay(0.2f);
        DelayAction d4 = Actions.delay(5f);
        final SequenceAction seq = Actions.sequence(d1, scrollABitRight, d2, scrollABitLeft, d3, scrollABitRight2, d4);
        scrollAffordanceAction = Actions.forever(seq);
        stage.addAction(scrollAffordanceAction);
    }

    private RepeatAction scrollAffordanceAction;
	private boolean isAffordanceEnabled = true;

    private void enableScrollAffordance() {
    	// re-enabling still causes some lags when combined with re-centering 
//    	if (!isAffordanceEnabled) {
//        	scrollAffordanceAction.restart();
//        	isAffordanceEnabled = true;
//    	}
    }

    private void disableScrollAffordance() {
    	if (isAffordanceEnabled) {
        	scrollAffordanceAction.finish();
        	// after finished it will get removed from stage, but is also not need anymore, since the user is obviously able to swipe
        	isAffordanceEnabled = false;
    	}
    }
    @Override
    public void render(float delta) {
        lifetime += delta;

        // Clear the screen
        Gdx.gl.glClearColor(0.9f, 0.9f, 0.9f, 1);
        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);

        // Update the stage and draw the actors
        stage.act(delta);
        stage.draw();

        // Check if the user is interacting with the scroll pane
        if (!Gdx.input.isTouched()) {
            // If the user is not interacting, activate the scroll affordance animation
            enableScrollAffordance();

            // Center the closest button group
            if (! scrollPane.isDragging() && ! scrollPane.isFlinging() && ! scrollPane.isPanning())
            	centerClosestButtonGroup();
        } else {
            // If the user is interacting, deactivate the scroll affordance animation
            disableScrollAffordance();
            
            hasCentered = false;
        }

        if (lifetime > 0.8f && lifetime < 1f) {
            scrollPane.setVelocityX(1);
            scrollPane.setScrollPercentX(0.5f); // Scroll to the center initially
        }
    }
    
    private boolean hasCentered = false;

    private void centerClosestButtonGroup() {
    	if (hasCentered)
    		return;
    	hasCentered = true;
        float scrollX = scrollPane.getScrollX();
        float viewportWidth = viewport.getWorldWidth();

        float minDistance = Float.MAX_VALUE;
        int closestIndex = -1;

        for (int i = 1; i < horizontalGroup.getChildren().size-1; i++) {
            Actor child = horizontalGroup.getChildren().get(i);
            float childX = child.getX();
            float childWidth = child.getWidth();
            float childCenterX = childX + childWidth / 2;
            float distance = Math.abs(childCenterX - (scrollX + viewportWidth / 2));

            if (distance < minDistance) {
                minDistance = distance;
                closestIndex = i;
            }
        }

        if (closestIndex != -1) {
            Actor closestChild = horizontalGroup.getChildren().get(closestIndex);
            float closestChildX = closestChild.getX();
            float closestChildWidth = closestChild.getWidth();
            float closestChildCenterX = closestChildX + closestChildWidth / 2;
            float targetScrollX = closestChildCenterX - viewportWidth / 2;

            scrollPane.setScrollX(targetScrollX);
        }
    }


    @Override
    public void resize(int width, int height) {
        viewport.update(width, height, true);
    }

    @Override
    public void dispose() {
        // TODO dispose texture of buttons too
        stage.dispose();
    }
}
